from .db_engine_plugin_interface import *
from .task_engine_plugin_interface import *
from .logger_interface import *
from .settings_interface import *
from .embed_interface import *
from .loader_interface import *



from dataclasses import Field
from datetime import datetime
from typing import Optional
from pydantic import BaseModel, field_serializer, Field


class Tenant(BaseModel):
    tenant_id: str = Field(None, description="tenant id")
    tenant_name: str = Field(None, description="tenant name")
    email: str = Field(..., description="email")
    user_id: Optional[str] = Field(None, description="user id")
    secret_key: str = Field(..., description="secret_key")
    is_active: bool = Field(True, description="is active")
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now().isoformat(), description="creation time"
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now().isoformat(), description="update time"
    )

    @field_serializer("created_at")
    def serialize_created_at(self, created_at: Optional[datetime]):
        return created_at.isoformat() if created_at else None

    @field_serializer("updated_at")
    def serialize_updated_at(self, updated_at: Optional[datetime]):
        return updated_at.isoformat() if updated_at else None

    def update(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.updated_at = datetime.now().isoformat()
        return self



from dataclasses import Field
from datetime import datetime
from enum import Enum
import hashlib
from typing import List, Optional

from pydantic import BaseModel, field_serializer, Field


def calculate_sha256(text):
    # 将文本转换为 UTF-8 编码的字节
    text_bytes = text.encode("utf-8")

    # 创建 SHA-256 哈希对象
    sha256_hash = hashlib.sha256()

    # 更新哈希对象
    sha256_hash.update(text_bytes)

    # 返回十六进制形式的哈希值
    return sha256_hash.hexdigest()


class KnowledgeSourceType(str, Enum):
    GITHUB_REPO = "github_repo"
    S3 = "S3"
    TEXT = "text"


class KnowledgeType(str, Enum):
    TEXT = "text"
    MARKDOWN = "markdown"
    HTML = "html"
    JSON = "json"
    PDF = "pdf"
    CSV = "csv"
    DOCX = "docx"
    PPTX = "pptx"
    IMAGE = "image"


class EmbeddingModelEnum(str, Enum):
    OPENAI = "openai"
    qwen = "qwen"


class KnowledgeSplitConfig(BaseModel):
    separators: List[str] = Field(None, description="separators")
    split_regex: str = Field(description="split regex")
    chunk_size: int = Field(2000, description="chunk size")
    chunk_overlap: int = Field(200, description="chunk overlap")


class KnowledgeCreate(BaseModel):
    """
    KnowledgeCreate model for creating knowledge resources.
    Attributes:
        knowledge_type (ResourceType): Type of knowledge resource.
        space_id (str): Space ID, example: petercat bot ID.
        knowledge_name (str): Name of the knowledge resource.
        file_sha (Optional[str]): SHA of the file.
        file_size (Optional[int]): Size of the file.
        split_config (Optional[dict]): Configuration for splitting the knowledge.
        source_data (Optional[str]): Source data of the knowledge.
        source_url (Optional[str]): URL of the source.
        auth_info (Optional[str]): Authentication information.
        embedding_model_name (Optional[str]): Name of the embedding model.
        metadata (Optional[dict]): Additional metadata.
    """

    source_type: KnowledgeSourceType = Field(description="source type")
    knowledge_type: KnowledgeType = Field(
        None, description="type of knowledge resource"
    )
    space_id: str = Field(None, description="space id, example: petercat bot id")
    knowledge_name: str = Field(None, description="name of the knowledge resource")
    file_sha: Optional[str] = Field(None, description="SHA of the file")
    file_size: Optional[int] = Field(None, description="size of the file")
    split_config: Optional[dict] = Field(
        None, description="configuration for splitting the knowledge"
    )
    source_data: Optional[str] = Field(None, description="source data of the knowledge")
    source_url: Optional[str] = Field(
        None,
        description="URL of the source",
        pattern=r"^(https?|ftp)://[^\s/$.?#].[^\s]*$",
    )
    auth_info: Optional[str] = Field(None, description="authentication information")
    embedding_model_name: Optional[EmbeddingModelEnum] = Field(
        EmbeddingModelEnum.OPENAI, description="name of the embedding model"
    )
    metadata: Optional[dict] = Field(None, description="additional metadata")

    @field_serializer("knowledge_type")
    def serialize_knowledge_type(self, knowledge_type):
        if isinstance(knowledge_type, KnowledgeType):
            return knowledge_type.value
        return str(knowledge_type)

    @field_serializer("source_type")
    def serialize_source_type(self, source_type):
        if isinstance(source_type, KnowledgeSourceType):
            return source_type.value
        return str(source_type)

    @field_serializer("embedding_model_name")
    def serialize_embedding_model_name(self, embedding_model_name):
        if isinstance(embedding_model_name, EmbeddingModelEnum):
            return embedding_model_name.value
        return str(embedding_model_name)


class KnowledgeResponse(BaseModel):
    knowledge_id: Optional[str] = None
    space_id: str = None
    knowledge_name: str
    sha: Optional[str] = None
    source_url: Optional[str] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    space_id: str
    tenant_id: str


class Knowledge(KnowledgeCreate):
    """
    Knowledge model class that extends KnowledgeCreate.
    Attributes:
        knowledge_id (str): Knowledge ID.
        tenant_id (str): Tenant ID.
        created_at (Optional[datetime]): Creation time, defaults to current time in ISO format.
        updated_at (Optional[datetime]): Update time, defaults to current time in ISO format.
    Methods:
        serialize_created_at(created_at: Optional[datetime]) -> Optional[str]:
            Serializes the created_at attribute to ISO format.
        serialize_updated_at(updated_at: Optional[datetime]) -> Optional[str]:
            Serializes the updated_at attribute to ISO format.
        update(**kwargs) -> 'Knowledge':
            Updates the attributes of the instance with the provided keyword arguments and sets updated_at to the current time.
    """

    knowledge_id: str = Field(None, description="knowledge id")
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(), description="creation time"
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(), description="update time"
    )
    tenant_id: str = Field(..., description="tenant id")

    def __init__(self, **data):
        super().__init__(**data)
        if (
            self.source_data is not None
            and self.file_sha == None
            and self.source_type == KnowledgeSourceType.TEXT
        ):
            self.file_sha = calculate_sha256(self.source_data)
            self.file_size = len(self.source_data.encode("utf-8"))

    @field_serializer("created_at")
    def serialize_created_at(self, created_at: Optional[datetime]):
        return created_at.isoformat() if created_at else None

    @field_serializer("updated_at")
    def serialize_updated_at(self, updated_at: Optional[datetime]):
        return updated_at.isoformat() if updated_at else None

    def update(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.updated_at = datetime.now().isoformat()
        return self



import importlib
import os
from pathlib import Path
from typing import Type, Union

from whiskerrag_types.interface.embed_interface import BaseEmbedding
from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.model.knowledge import KnowledgeSourceType, KnowledgeType

RegisterType = Union[KnowledgeSourceType, KnowledgeType]
RegisterItem = Union[BaseLoader, BaseEmbedding]

_registry = {}
_loaded_packages = set()


def register(register_type: Union[KnowledgeSourceType, KnowledgeType]):
    def decorator(cls):
        cls._is_register_item = True
        if not issubclass(cls, (BaseLoader, BaseEmbedding)):
            raise TypeError(
                f"Class {cls.__name__} must inherit from BaseLoader or BaseEmbedding"
            )
        _registry[register_type] = cls
        return cls

    return decorator


def init_register(package_name: str = "whiskerrag_utils") -> None:
    if package_name in _loaded_packages:
        return
    try:
        package = importlib.import_module(package_name)
        package_path = Path(package.__file__).parent
        current_file = Path(__file__).name
        for root, _, files in os.walk(package_path):
            for file in files:
                # 跳过当前文件和非 Python 文件
                if file == current_file or not file.endswith(".py"):
                    continue
                # 构建模块路径
                module_name = (
                    Path(root, file)
                    .relative_to(package_path)
                    .with_suffix("")
                    .as_posix()
                    .replace("/", ".")
                )

                if module_name == "__init__":
                    continue
                try:
                    module = importlib.import_module(f"{package_name}.{module_name}")
                    for attr_name in dir(module):
                        attr = getattr(module, attr_name)
                        if getattr(attr, "_is_register_item", False):
                            _registry[attr_name] = attr

                except ImportError as e:
                    print(f"Error importing module {module_name}: {e}")

        _loaded_packages.add(package_name)

    except ImportError as e:
        print(f"Error importing package {package_name}: {e}")


def get_register(loader_type: RegisterType) -> Type[RegisterItem]:
    loader = _registry.get(loader_type)
    if loader is None:
        raise KeyError(f"No loader registered for type: {loader_type}")
    return loader



from abc import ABC, abstractmethod
from typing import List
from langchain_core.documents import Document

from ..model import Knowledge, Chunk


class BaseEmbedding(ABC):
    @abstractmethod
    async def embed(
        self, knowledge: Knowledge, documents: List[Document]
    ) -> List[Chunk]:
        pass



from abc import ABC, abstractmethod
from typing import List

from ..interface import (
    SettingsInterface,
    LoggerManagerInterface,
)
from ..model import Tenant, KnowledgeCreate, Knowledge, Task


class TaskEnginPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface

    def __init__(
        self,
        logger: LoggerManagerInterface,
        settings: SettingsInterface,
    ):
        try:
            logger.info("TaskEngine plugin is initializing...")
            self.settings = settings
            self.logger = logger
            self.init()
            logger.info("TaskEngine plugin is initialized")
        except Exception as e:
            logger.error(f"TaskEngine plugin init error: {e}")

    @abstractmethod
    def init(self):
        pass

    @abstractmethod
    async def gen_knowledge_list(
        self, user_input: List[KnowledgeCreate], tenant: Tenant
    ) -> List[Task]:
        pass

    @abstractmethod
    async def init_task_from_knowledge(
        self, knowledge_list: List[Knowledge], tenant: Tenant
    ) -> List[Task]:
        pass

    @abstractmethod
    async def execute_task(self, task_id: str) -> List[Task]:
        pass

    @abstractmethod
    async def batch_execute_task(
        self, task_list: List[Task], knowledge_list: List[Knowledge]
    ) -> List[Task]:
        pass



from dataclasses import Field
from datetime import datetime
from enum import Enum
from typing import Optional
from pydantic import BaseModel, field_serializer, Field


class TaskStatus(str, Enum):
    PENDING = "pending"
    RUNNING = "running"
    SUCCESS = "success"
    FAILED = "failed"


class Task(BaseModel):
    """
    Task model representing a task entity with various attributes.
    Attributes:
        task_id (str): Task ID.
        status (TaskStatus): Status of the task.
        knowledge_id (str): File source information.
        space_id (str): Space ID.
        user_id (Optional[str]): User ID.
        tenant_id (str): Tenant ID.
        created_at (Optional[datetime]): Creation time, defaults to current time.
        updated_at (Optional[datetime]): Update time, defaults to current time.
    Methods:
        serialize_created_at(created_at: Optional[datetime]) -> str:
            Serializes the created_at attribute to ISO format.
        serialize_updated_at(updated_at: Optional[datetime]) -> str:
            Serializes the updated_at attribute to ISO format.
        update(**kwargs) -> Task:
            Updates the task attributes with provided keyword arguments and sets updated_at to current time.
    """

    task_id: str = Field(None, description="task id")
    status: TaskStatus = Field(None, description="task status")
    knowledge_id: str = Field(..., description="file source info")
    space_id: str = Field(..., description="space id")
    user_id: Optional[str] = Field(None, description="user id")
    tenant_id: str = Field(..., description="tenant id")
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(), description="creation time"
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(), description="update time"
    )

    @field_serializer("status")
    def serialize_status(self, status: TaskStatus):
        return status.value if isinstance(status, TaskStatus) else str(status)

    @field_serializer("created_at")
    def serialize_created_at(self, created_at: Optional[datetime]):
        return created_at.isoformat() if created_at else None

    @field_serializer("updated_at")
    def serialize_updated_at(self, updated_at: Optional[datetime]):
        return updated_at.isoformat() if updated_at else None

    def update(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.updated_at = datetime.now().isoformat()
        return self

    class Config:
        extra = "ignore"



import base64
from typing import List, Optional
from github import Github
from langchain_core.documents import Document

from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.model.knowledge import Knowledge, KnowledgeSourceType
from whiskerrag_utils.registry import register


@register(KnowledgeSourceType.GITHUB_REPO)
class GithubFileLoader(BaseLoader):
    knowledge: Knowledge
    path: str
    mode: str
    url: str
    branch: str
    repo_name: str
    size: int
    sha: str
    commit_id: Optional[str]
    github: Github

    def __init__(
        self,
        knowledge: Knowledge,
    ):
        self.knowledge = knowledge
        if not all(
            key in knowledge.metadata for key in ["repo_name", "path", "branch"]
        ):
            raise ValueError("metadata must contain 'repo_name', 'path', and 'branch'")
        if not knowledge.auth_info:
            raise ValueError("auth_info is required")
        self.github = Github(knowledge.auth_info)
        self.repo_name = knowledge.metadata.get("repo_name")
        self.repo = self.github.get_repo(self.repo_name)
        self.path = knowledge.metadata.get("path")
        self.branch = knowledge.metadata.get("branch")
        self.commit_id = knowledge.metadata.get("commit_id")

    def _get_commit_id_by_branch(self, branch: str) -> str:
        branch_info = self.github.get_branch(branch)
        return branch_info.commit.sha

    def _get_file_content_by_path(
        self,
    ) -> str:
        file_content = (
            self.repo.get_contents(self.path, ref=self.commit_id)
            if self.commit_id
            else self.repo.get_contents(self.path)
        )
        if isinstance(file_content, list):
            print("[warn]file_content is a list")
            file_content = file_content[0]
        self.sha = file_content.sha
        self.size = file_content.size
        return base64.b64decode(file_content.content).decode("utf-8")

    async def load(self) -> List[Document]:
        content = self._get_file_content_by_path()
        metadata = {
            **self.knowledge.metadata,
            "sha": self.sha,
            "size": self.size,
        }
        return [Document(page_content=content, metadata=metadata)]



from abc import ABC, abstractmethod


class LoggerManagerInterface(ABC):
    @abstractmethod
    def get_logger(self):
        pass

    @abstractmethod
    def info(self, message: str, *args, **kwargs):
        pass

    @abstractmethod
    def error(self, message: str, *args, **kwargs):
        pass

    @abstractmethod
    def debug(self, message: str, *args, **kwargs):
        pass

    @abstractmethod
    def warning(self, message: str, *args, **kwargs):
        pass
